package cn.com.bril.androidocr.studio.ui.activity;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.support.annotation.IdRes;
import android.support.annotation.NonNull;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v4.view.ViewPager;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.RadioGroup;
import android.widget.TextView;

import com.bril.baoding.studio.aidl.GetUserid;
import com.scanlibrary.ScanConstants;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import cn.com.bril.androidocr.studio.OcrApplication;
import cn.com.bril.androidocr.studio.R;
import cn.com.bril.androidocr.studio.configs.Configs;
import cn.com.bril.androidocr.studio.ui.adapter.TabFragmentPagerAdapter;
import cn.com.bril.androidocr.studio.ui.fragment.ExportFragment;
import cn.com.bril.androidocr.studio.ui.fragment.ShareFragment;

public class MainActivity extends BaseActivity {


    @BindView(R.id.layout_1)
    LinearLayout layout1;
    @BindView(R.id.layout_2)
    LinearLayout layout2;
    @BindView(R.id.layout_3)
    LinearLayout layout3;
    @BindView(R.id.viewPager)
    ViewPager viewPager;
    private String picPath;
    @NonNull
    @BindView(R.id.rightTv)
    TextView rightTv;
    @BindView(R.id.fabbtn)
    FloatingActionButton fabbtn;
    @BindView(R.id.topTitle)
    TextView topTitle;
    @BindView(R.id.left_tv)
    TextView leftTv;
    @BindView(R.id.group)
    RadioGroup group;

    private int type;//获取图片类型
    private GetUserid mService;
    private List<Fragment> fragmentList;
    private ServiceConnection mServiceConnection = new ServiceConnection() {

        @Override
        public void onServiceDisconnected(ComponentName arg0) {
            mService = null;
        }

        @Override
        public void onServiceConnected(ComponentName arg0, IBinder arg1) {
            mService = GetUserid.Stub.asInterface(arg1);
            try {
                Configs.USER_ID = mService.getUserid() + "";
                Configs.USER_NAME = mService.getUsername();

            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
    };


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        ButterKnife.bind(this);
        OcrApplication.getInstance().addActivity(this);

//        bindService();

        initViews();

        initEvents();

    }

    private void bindService() {
        Intent intent = new Intent("com.xtpro.studio.service.AIDLService");
        intent.setPackage("com.bril.baoding.studio");
        bindService(intent, mServiceConnection, Context.BIND_AUTO_CREATE);
    }

    private void initViews() {
        rightTv.setText("导入");
        topTitle.setText("OCR识别");

        fragmentList=new ArrayList<>();
        fragmentList.add(new ExportFragment());
        fragmentList.add(new ShareFragment());
        TabFragmentPagerAdapter adapter=new TabFragmentPagerAdapter(getSupportFragmentManager(),fragmentList);
        viewPager.setAdapter(adapter);

    }

    private void setImageGone() {
        layout1.setVisibility(View.GONE);
        layout2.setVisibility(View.INVISIBLE);
        layout3.setVisibility(View.INVISIBLE);
    }

    private void initEvents() {
        group.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, @IdRes int checkedId) {

                String tag = null;
                switch (checkedId) {
                    case R.id.radio_1:
                        tag = "生成记录";
                        break;
                    case R.id.radio_2:
                        tag = "导出记录";
                      setViewShow(0);
                        break;
                    case R.id.radio_3:
                        tag = "分享记录";
                      setViewShow(1);
                        break;
                }
                topTitle.setText(tag);
            }
        });

        viewPager.setOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

            }

            @Override
            public void onPageSelected(int position) {
               setViewShow(position);
            }

            @Override
            public void onPageScrollStateChanged(int state) {

            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    public void setViewShow(int position){
        setImageGone();
        switch (position){
            case 0:
                layout2.setVisibility(View.VISIBLE);
                break;
            case 1:
                layout3.setVisibility(View.VISIBLE);
                break;
        }
        viewPager.setCurrentItem(position);
    }

    @OnClick({R.id.rightTv, R.id.fabbtn})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.rightTv:
                type = ScanConstants.OPEN_MEDIA;
                break;
            case R.id.fabbtn:
                type = ScanConstants.OPEN_CAMERA;
                break;
        }
        Intent intent = new Intent(this, RecognizeActivity.class);
        intent.putExtra("type", type);
        startActivity(intent);
    }

    @Override
    public void onBackPressed() {

        OcrApplication.getInstance().exit();
        super.onBackPressed();
    }
}
